@@ -1,5 +1,6 @@
 use scraper::{Html, Selector};
 use url::Url;
+use crate::utils::{fetch_page_html, Client};
 
 /// Represent a page visited. This page contains HTML scraped with [scraper](https://crates.io/crates/scraper).
 ///
@@ -10,12 +11,16 @@ pub struct Page {
     url: String,
     /// HTML parsed with [scraper](https://crates.io/crates/scraper) lib
     html: String,
+    /// Base absolute url for domain
+    base: Url
 }
 
 impl Page {
     /// Instanciate a new page and start to scrape it.
-    pub fn new(url: &str, html: &str) -> Self {
-        Page::build(url, html)
+    pub fn new(url: &str, client: &Client) -> Self {
+        let html = fetch_page_html(&url, &client);
+
+        Page::build(url, &html)
     }
 
     /// Instanciate a new page without scraping it (used for testing purposes)
@@ -23,6 +28,7 @@ impl Page {
         Self {
             url: url.to_string(),
             html: html.to_string(),
+            base: Url::parse(&url).expect("Invalid page URL")
         }
     }
 
@@ -62,57 +68,55 @@ impl Page {
         .unwrap()
     }
 
-    /// Find all href links and return them
-    pub fn links(&self) -> Vec<String> {
+    /// Find all href links and return them: this also clears the set html for the page
+    pub fn links(&mut self) -> Vec<String> {
         let selector = self.get_page_selectors(&self.url);
         let html = self.get_html();
 
+        self.html.clear();
+        
         html.select(&selector)
             .map(|a| self.abs_path(a.value().attr("href").unwrap_or_default()).to_string())
             .collect()
     }
 
     fn abs_path(&self, href: &str) -> Url {
-        let base = Url::parse(&self.url.to_string()).expect("Invalid page URL");
-        let mut joined = base.join(href).unwrap_or(base);
+        let mut joined = self.base.join(href).unwrap_or(Url::parse(&self.url.to_string()).expect("Invalid page URL"));
 
         joined.set_fragment(None);
 
         joined
     }
 }
-
 #[test]
 fn parse_links() {
-    use crate::utils::{fetch_page_html, Client};
     let client = Client::builder()
         .user_agent("spider/1.1.2")
         .build()
         .unwrap();
 
     let link_result = "https://choosealicense.com/";
-    let html = fetch_page_html(&link_result, &client);
-    let page: Page = Page::new(&link_result, &html);
+    let mut page: Page = Page::new(&link_result, &client);
+
+    let links = page.links();
 
     assert!(
-        page.links()
+        links
             .contains(&"https://choosealicense.com/about/".to_string()),
         "Could not find {}. Theses URLs was found {:?}",
         page.url,
-        page.links()
+        &links
     );
 }
 
 #[test]
 fn test_abs_path() {
-    use crate::utils::{fetch_page_html, Client};
     let client = Client::builder()
         .user_agent("spider/1.1.2")
         .build()
         .unwrap();
     let link_result = "https://choosealicense.com/";
-    let html = fetch_page_html(&link_result, &client);
-    let page: Page = Page::new(&link_result, &html);
+    let page: Page = Page::new(&link_result, &client);
 
     assert_eq!(
         page.abs_path("/page"),
@@ -138,4 +142,4 @@ fn test_abs_path() {
         page.abs_path("tel://+212 3456"),
         Url::parse("https://choosealicense.com/").unwrap()
     );
-}
+}
\ No newline at end of file
