@@ -1,7 +1,7 @@
 use crate::black_list::contains;
 use crate::configuration::Configuration;
 use crate::page::Page;
-use crate::utils::{fetch_page_html, Client};
+use crate::utils::{Client};
 
 use rayon::ThreadPool;
 use rayon::ThreadPoolBuilder;
@@ -11,6 +11,7 @@ use std::collections::HashSet;
 use std::{sync, thread, time::Duration};
 use reqwest::header::CONNECTION;
 use reqwest::header;
+use sync::mpsc::{channel, Sender, Receiver};
 
 /// Represent a website to scrawl. To start crawling, instanciate a new `struct` using
 /// <pre>
@@ -43,6 +44,8 @@ pub struct Website<'a> {
     pub page_store_ignore: bool,
 }
 
+type Message = (Page, Vec<String>);
+
 impl<'a> Website<'a> {
     /// Initialize Website object with a start link to scrawl.
     pub fn new(domain: &str) -> Self {
@@ -105,13 +108,12 @@ impl<'a> Website<'a> {
     pub fn crawl(&mut self) {
         self.configure_robots_parser();
         let client = self.configure_http_client(None);
-        let delay = self.get_delay();
         let on_link_find_callback = self.on_link_find_callback;
         let pool = self.create_thread_pool();
         
         // crawl while links exists
         while !self.links.is_empty() {
-            let (tx, rx) = sync::mpsc::channel();
+            let (tx, rx): (Sender<Message>, Receiver<Message>) = channel();
 
             for link in self.links.iter() {
                 if !self.is_allowed(link) {
@@ -124,10 +126,13 @@ impl<'a> Website<'a> {
                 let tx = tx.clone();
                 let cx = client.clone();
 
+                if self.configuration.delay > 0 {
+                    thread::sleep(self.get_delay());
+                }
+                
                 pool.spawn(move || {
                     let link_result = on_link_find_callback(link);
-                    let html = fetch_page_html(&link_result, &cx);
-                    let page = Page::new(&link_result, &html);
+                    let mut page = Page::new(&link_result, &cx);
                     let links = page.links();
 
                     tx.send((page, links)).unwrap();
@@ -148,12 +153,9 @@ impl<'a> Website<'a> {
                     self.pages.push(page);
                 }
 
-                if self.configuration.delay > 0 {
-                    thread::sleep(delay);
-                }
             });
 
-            self.links = new_links;
+            self.links = new_links.difference(&self.links_visited).cloned().collect();
         }
     }
 
